//* G.U.N.T. FL 151 Balze Edition *//

#ifndef F_CPU
#define F_CPU 6000000
#endif

#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <math.h>

#include "peter_fleury_usart\uart.h"


/* define CPU frequency in Mhz here if not defined in Makefile */
#ifndef F_CPU
#define F_CPU 6000000UL
#endif

/* 9600 baud */
#define UART_BAUD_RATE      125000      



#include "peter_fleury_usart\uart.h"
#include "TI_ADS1234.h"
// #include "FL151_balze_edition.h"


#define ADS_1_DRDY_P_IN		&PIND 
#define ADS_1_SCLK_PORT		&PORTC
#define ADS_1_DRDY_pin		3
#define ADS_1_SCLK_pin		1

#define ADS_2_DRDY_P_IN		&PIND 
#define ADS_2_SCLK_PORT		&PORTC
#define ADS_2_DRDY_pin		4
#define ADS_2_SCLK_pin		0

#define ADS_3_DRDY_P_IN		&PIND 
#define ADS_3_SCLK_PORT		&PORTC
#define ADS_3_DRDY_pin		5
#define ADS_3_SCLK_pin		2

#define ADS_4_DRDY_P_IN		&PINB 
#define ADS_4_SCLK_PORT		&PORTB
#define ADS_4_DRDY_pin		2
#define ADS_4_SCLK_pin		1

#define ADS_A0_PORT			PORTB
#define ADS_A0_pin			0
#define ADS_A1_PORT			PORTD
#define ADS_A1_pin			7


volatile uint32_t Millisekunden = 0;

ISR(TIMER1_COMPA_vect)
{
	Millisekunden ++;
}

int main ()
{
	uint8_t ADS1 = 0;	
	uint8_t ADS2 = 1;	
	uint8_t ADS3 = 1;	
	uint8_t ADS4 = 1;	

	char test = 0;

	uint8_t multiplexer = 0;
	uint8_t addition = 0;
	int32_t temp = 0;
	int32_t data_array[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	char rs232_array[65] = {'.','.','.','.','.','.','.','.','.','.','.','.','.','.','P','l','e','a','s','e',' ','s','e','n','d',' ','t','h','i','s',' ','d','a','t','a','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.',0};


	TCCR1B = _BV(WGM12) | _BV(CS11) | _BV(CS10);
	TIMSK1 = _BV(OCIE1A);
	
	OCR1A = 94;
	TCNT1 = 0x00;

	
	
	DDRB |= (1 << PINB3);
	DDRB |= (1 << PINB4);
	DDRB |= (1 << PINB5);

	DDRC |= (1 << PINC1);		// SCLK ADS_1
	DDRC |= (1 << PINC0);		// SCLK ADS_2
	DDRC |= (1 << PINC2);		// SCLK ADS_3
	DDRB |= (1 << PINB1);		// SCLK ADS_4

	DDRB |= (1 << PINB0);		// ADS A0
	DDRD |= (1 << PIND7);		// ADS A1

	DDRD |= (1 << PIND6);		// SPEED !

	PORTD |= (1<<PIND6);		// full speed (80S/sec)


	uart_init(UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU));
	
	sei();

	ADS_A0_PORT &= ~(1<<ADS_A0_pin);	// A0 lschen
	ADS_A1_PORT &= ~(1<<ADS_A1_pin);	// A1 lschen
	
	
	_delay_ms (100);


	calibrate(ADS_1_DRDY_P_IN, ADS_1_SCLK_PORT, ADS_1_DRDY_pin, ADS_1_SCLK_pin);
	calibrate(ADS_2_DRDY_P_IN, ADS_2_SCLK_PORT, ADS_2_DRDY_pin, ADS_2_SCLK_pin);
	calibrate(ADS_3_DRDY_P_IN, ADS_3_SCLK_PORT, ADS_3_DRDY_pin, ADS_3_SCLK_pin);
	calibrate(ADS_4_DRDY_P_IN, ADS_4_SCLK_PORT, ADS_4_DRDY_pin, ADS_4_SCLK_pin);



	_delay_ms (100);


	for (;;)
	{


		switch (multiplexer) // Reihenfolge des Scans ist 1,3,2,4 wegen des aufbaus des ADS
		{
			case 0 : 	ADS_A0_PORT &= ~(1<<ADS_A0_pin);	// A0 lschen
						ADS_A1_PORT &= ~(1<<ADS_A1_pin);	// A1 lschen
						break;
			case 1 : 	ADS_A0_PORT &= ~(1<<ADS_A0_pin);	// A0 lschen
						ADS_A1_PORT |= (1<<ADS_A1_pin);	// A1 setzen
						break;
			case 2 : 	ADS_A0_PORT |= (1<<ADS_A0_pin);	// A0 setzen
						ADS_A1_PORT &= ~(1<<ADS_A1_pin);	// A1 lschen
						break;
						
			case 3 : 	ADS_A0_PORT |= (1<<ADS_A0_pin);	// A0 setzen
						ADS_A1_PORT |= (1<<ADS_A1_pin);	// A1 setzen
						break;
			default :	break;
		}


		_delay_ms (40);


		addition = multiplexer*4;

		temp = read_ADS1234(ADS_1_DRDY_P_IN, ADS_1_SCLK_PORT, ADS_1_DRDY_pin, ADS_1_SCLK_pin);
		data_array[addition] = temp;

		temp = read_ADS1234(ADS_2_DRDY_P_IN, ADS_2_SCLK_PORT, ADS_2_DRDY_pin, ADS_2_SCLK_pin);
		data_array[addition+1] = temp;

		temp = read_ADS1234(ADS_3_DRDY_P_IN, ADS_3_SCLK_PORT, ADS_3_DRDY_pin, ADS_3_SCLK_pin);
		data_array[addition+2] = temp;

		temp = read_ADS1234(ADS_4_DRDY_P_IN, ADS_4_SCLK_PORT, ADS_4_DRDY_pin, ADS_4_SCLK_pin);
		data_array[addition+3] = temp;

		switch (multiplexer)
		{
			case 0 :	PORTB |= (1<<PINB4);
						PORTB |= (1<<PINB3);
						break;
			case 1 :	PORTB &= ~(1<<PINB4);
						PORTB |= (1<<PINB3);
						break;
			case 2 :	PORTB |= (1<<PINB4);
						PORTB &= ~(1<<PINB3);
						break;
			case 3 :	PORTB &= ~(1<<PINB4);
						PORTB &= ~(1<<PINB3);
						break;
			default :	break;
		}

		if (++multiplexer >= 2)// nur 2 Kanaele aller Baenke 4)	//bertragen, von vorne anfangen
		{
			PORTB &= ~(1<<PINB5);
			multiplexer = 0;
			test++;
			for (uint8_t i = 0; i < 16; i++)
			{
				addition = i*4;
				rs232_array[addition] = (char)(data_array[i]>>16); 	
				rs232_array[addition+1] = (char)(data_array[i]>>8); 	
				rs232_array[addition+2] = (char)(data_array[i]); 	
				rs232_array[addition+3] = 0; 	
			}
			for (uint8_t i = 0; i < 66; i++)
				uart_putc(rs232_array[i]);
		}
		else
			PORTB |= (1<<PINB5);

	}
		
}







    


